@@ -1,5 +1,11 @@
 
 package Moose::Meta::Class;
+BEGIN {
+  $Moose::Meta::Class::AUTHORITY = 'cpan:STEVAN';
+}
+BEGIN {
+  $Moose::Meta::Class::VERSION = '2.0200';
+}
 
 use strict;
 use warnings;
@@ -10,11 +16,7 @@ use Carp qw( confess );
 use Data::OptList;
 use List::Util qw( first );
 use List::MoreUtils qw( any all uniq first_index );
-use Scalar::Util 'weaken', 'blessed';
-
-our $VERSION   = '1.09';
-$VERSION = eval $VERSION;
-our $AUTHORITY = 'cpan:STEVAN';
+use Scalar::Util 'blessed';
 
 use Moose::Meta::Method::Overridden;
 use Moose::Meta::Method::Augmented;
@@ -22,75 +24,82 @@ use Moose::Error::Default;
 use Moose::Meta::Class::Immutable::Trait;
 use Moose::Meta::Method::Constructor;
 use Moose::Meta::Method::Destructor;
+use Moose::Meta::Method::Meta;
+use Moose::Util;
+use Class::MOP::MiniTrait;
 
 use base 'Class::MOP::Class';
 
+Class::MOP::MiniTrait::apply(__PACKAGE__, 'Moose::Meta::Object::Trait');
+
 __PACKAGE__->meta->add_attribute('roles' => (
     reader  => 'roles',
-    default => sub { [] }
+    default => sub { [] },
+    Class::MOP::_definition_context(),
 ));
 
 __PACKAGE__->meta->add_attribute('role_applications' => (
     reader  => '_get_role_applications',
-    default => sub { [] }
+    default => sub { [] },
+    Class::MOP::_definition_context(),
 ));
 
 __PACKAGE__->meta->add_attribute(
     Class::MOP::Attribute->new('immutable_trait' => (
         accessor => "immutable_trait",
         default  => 'Moose::Meta::Class::Immutable::Trait',
+        Class::MOP::_definition_context(),
     ))
 );
 
 __PACKAGE__->meta->add_attribute('constructor_class' => (
     accessor => 'constructor_class',
     default  => 'Moose::Meta::Method::Constructor',
+    Class::MOP::_definition_context(),
 ));
 
 __PACKAGE__->meta->add_attribute('destructor_class' => (
     accessor => 'destructor_class',
     default  => 'Moose::Meta::Method::Destructor',
+    Class::MOP::_definition_context(),
 ));
 
 __PACKAGE__->meta->add_attribute('error_class' => (
     accessor => 'error_class',
     default  => 'Moose::Error::Default',
+    Class::MOP::_definition_context(),
 ));
 
 sub initialize {
     my $class = shift;
-    my $pkg   = shift;
-    return Class::MOP::get_metaclass_by_name($pkg)
-        || $class->SUPER::initialize($pkg,
+    my @args = @_;
+    unshift @args, 'package' if @args % 2;
+    my %opts = @args;
+    my $package = delete $opts{package};
+    return Class::MOP::get_metaclass_by_name($package)
+        || $class->SUPER::initialize($package,
                 'attribute_metaclass' => 'Moose::Meta::Attribute',
                 'method_metaclass'    => 'Moose::Meta::Method',
                 'instance_metaclass'  => 'Moose::Meta::Instance',
-                @_
+                %opts,
             );
 }
 
-sub _immutable_options {
-    my ( $self, @args ) = @_;
-
-    $self->SUPER::_immutable_options(
-        inline_destructor => 1,
-
-        # Moose always does this when an attribute is created
-        inline_accessors => 0,
-
-        @args,
-    );
-}
-
 sub create {
-    my ($class, $package_name, %options) = @_;
+    my $class = shift;
+    my @args = @_;
+
+    unshift @args, 'package' if @args % 2 == 1;
+    my %options = @args;
 
     (ref $options{roles} eq 'ARRAY')
         || $class->throw_error("You must pass an ARRAY ref of roles", data => $options{roles})
             if exists $options{roles};
-    my $roles = delete $options{roles};
 
-    my $new_meta = $class->SUPER::create($package_name, %options);
+    my $package = delete $options{package};
+    my $roles   = delete $options{roles};
+
+    my $new_meta = $class->SUPER::create($package, %options);
 
     if ($roles) {
         Moose::Util::apply_all_roles( $new_meta, @$roles );
@@ -99,34 +108,62 @@ sub create {
     return $new_meta;
 }
 
-my %ANON_CLASSES;
+sub _meta_method_class { 'Moose::Meta::Method::Meta' }
 
-sub create_anon_class {
-    my ($self, %options) = @_;
-
-    my $cache_ok = delete $options{cache};
-
-    my $cache_key
-        = _anon_cache_key( $options{superclasses}, $options{roles} );
-
-    if ($cache_ok && defined $ANON_CLASSES{$cache_key}) {
-        return $ANON_CLASSES{$cache_key};
-    }
-
-    my $new_class = $self->SUPER::create_anon_class(%options);
-
-    $ANON_CLASSES{$cache_key} = $new_class
-        if $cache_ok;
-
-    return $new_class;
-}
+sub _anon_package_prefix { 'Moose::Meta::Class::__ANON__::SERIAL::' }
 
 sub _anon_cache_key {
-    # Makes something like Super::Class|Super::Class::2=Role|Role::1
-    return join '=' => (
-        join( '|', @{ $_[0]      || [] } ),
-        join( '|', sort @{ $_[1] || [] } ),
+    my $class = shift;
+    my %options = @_;
+
+    my $superclass_key = join('|',
+        map { $_->[0] } @{ Data::OptList::mkopt($options{superclasses} || []) }
     );
+
+    my $roles = Data::OptList::mkopt(($options{roles} || []), {
+        moniker  => 'role',
+        val_test => sub { ref($_[0]) eq 'HASH' },
+    });
+
+    my @role_keys;
+    for my $role_spec (@$roles) {
+        my ($role, $params) = @$role_spec;
+        $params = { %$params } if $params;
+
+        my $key = blessed($role) ? $role->name : $role;
+
+        if ($params && %$params) {
+            my $alias    = delete $params->{'-alias'}
+                        || delete $params->{'alias'}
+                        || {};
+            my $excludes = delete $params->{'-excludes'}
+                        || delete $params->{'excludes'}
+                        || [];
+            $excludes = [$excludes] unless ref($excludes) eq 'ARRAY';
+
+            if (%$params) {
+                warn "Roles with parameters cannot be cached. Consider "
+                   . "applying the parameters before calling "
+                   . "create_anon_class, or using 'weaken => 0' instead";
+                return;
+            }
+
+            my $alias_key = join('%',
+                map { $_ => $alias->{$_} } sort keys %$alias
+            );
+            my $excludes_key = join('%',
+                sort @$excludes
+            );
+            $key .= '<' . join('+', 'a', $alias_key, 'e', $excludes_key) . '>';
+        }
+
+        push @role_keys, $key;
+    }
+
+    my $role_key = join('|', sort @role_keys);
+
+    # Makes something like Super::Class|Super::Class::2=Role|Role::1
+    return join('=', $superclass_key, $role_key);
 }
 
 sub reinitialize {
@@ -135,8 +172,6 @@ sub reinitialize {
 
     my $meta = blessed $pkg ? $pkg : Class::MOP::class_of($pkg);
 
-    my $cache_key;
-
     my %existing_classes;
     if ($meta) {
         %existing_classes = map { $_ => $meta->$_() } qw(
@@ -148,30 +183,13 @@ sub reinitialize {
             destructor_class
             error_class
         );
-
-        $cache_key = _anon_cache_key(
-            [ $meta->superclasses ],
-            [ map { $_->name } @{ $meta->roles } ],
-        ) if $meta->is_anon_class;
     }
 
-    my $new_meta = $self->SUPER::reinitialize(
+    return $self->SUPER::reinitialize(
         $pkg,
         %existing_classes,
         @_,
     );
-
-    return $new_meta unless defined $cache_key;
-
-    my $new_cache_key = _anon_cache_key(
-        [ $meta->superclasses ],
-        [ map { $_->name } @{ $meta->roles } ],
-    );
-
-    delete $ANON_CLASSES{$cache_key};
-    $ANON_CLASSES{$new_cache_key} = $new_meta;
-
-    return $new_meta;
 }
 
 sub add_role {
@@ -255,14 +273,22 @@ sub new_object {
     my $params = @_ == 1 ? $_[0] : {@_};
     my $object = $self->SUPER::new_object($params);
 
+    $self->_call_all_triggers($object, $params);
+
+    $object->BUILDALL($params) if $object->can('BUILDALL');
+
+    return $object;
+}
+
+sub _call_all_triggers {
+    my ($self, $object, $params) = @_;
+
     foreach my $attr ( $self->get_all_attributes() ) {
 
         next unless $attr->can('has_trigger') && $attr->has_trigger;
 
         my $init_arg = $attr->init_arg;
-
         next unless defined $init_arg;
-
         next unless exists $params->{$init_arg};
 
         $attr->trigger->(
@@ -274,10 +300,250 @@ sub new_object {
             ),
         );
     }
+}
 
-    $object->BUILDALL($params) if $object->can('BUILDALL');
+sub _generate_fallback_constructor {
+    my $self = shift;
+    my ($class) = @_;
+    return $class . '->Moose::Object::new(@_)'
+}
 
-    return $object;
+sub _inline_params {
+    my $self = shift;
+    my ($params, $class) = @_;
+    return (
+        'my ' . $params . ' = ',
+        $self->_inline_BUILDARGS($class, '@_'),
+        ';',
+    );
+}
+
+sub _inline_BUILDARGS {
+    my $self = shift;
+    my ($class, $args) = @_;
+
+    my $buildargs = $self->find_method_by_name("BUILDARGS");
+
+    if ($args eq '@_'
+     && (!$buildargs or $buildargs->body == \&Moose::Object::BUILDARGS)) {
+        return (
+            'do {',
+                'my $params;',
+                'if (scalar @_ == 1) {',
+                    'if (!defined($_[0]) || ref($_[0]) ne \'HASH\') {',
+                        $self->_inline_throw_error(
+                            '"Single parameters to new() must be a HASH ref"',
+                            'data => $_[0]',
+                        ) . ';',
+                    '}',
+                    '$params = { %{ $_[0] } };',
+                '}',
+                'elsif (@_ % 2) {',
+                    'Carp::carp(',
+                        '"The new() method for ' . $class . ' expects a '
+                      . 'hash reference or a key/value list. You passed an '
+                      . 'odd number of arguments"',
+                    ');',
+                    '$params = {@_, undef};',
+                '}',
+                'else {',
+                    '$params = {@_};',
+                '}',
+                '$params;',
+            '}',
+        );
+    }
+    else {
+        return $class . '->BUILDARGS(' . $args . ')';
+    }
+}
+
+sub _inline_slot_initializer {
+    my $self  = shift;
+    my ($attr, $idx) = @_;
+
+    return (
+        '## ' . $attr->name,
+        $self->_inline_check_required_attr($attr),
+        $self->SUPER::_inline_slot_initializer(@_),
+    );
+}
+
+sub _inline_check_required_attr {
+    my $self = shift;
+    my ($attr) = @_;
+
+    return unless defined $attr->init_arg;
+    return unless $attr->can('is_required') && $attr->is_required;
+    return if $attr->has_default || $attr->has_builder;
+
+    return (
+        'if (!exists $params->{\'' . $attr->init_arg . '\'}) {',
+            $self->_inline_throw_error(
+                '"Attribute (' . quotemeta($attr->name) . ') is required"'
+            ) . ';',
+        '}',
+    );
+}
+
+# XXX: these two are duplicated from cmop, because we have to pass the tc stuff
+# through to _inline_set_value - this should probably be fixed, but i'm not
+# quite sure how. -doy
+sub _inline_init_attr_from_constructor {
+    my $self = shift;
+    my ($attr, $idx) = @_;
+
+    my @initial_value = $attr->_inline_set_value(
+        '$instance',
+        '$params->{\'' . $attr->init_arg . '\'}',
+        '$type_constraint_bodies[' . $idx . ']',
+        '$type_coercions[' . $idx . ']',
+        '$type_constraint_messages[' . $idx . ']',
+        'for constructor',
+    );
+
+    push @initial_value, (
+        '$attrs->[' . $idx . ']->set_initial_value(',
+            '$instance,',
+            $attr->_inline_instance_get('$instance'),
+        ');',
+    ) if $attr->has_initializer;
+
+    return @initial_value;
+}
+
+sub _inline_init_attr_from_default {
+    my $self = shift;
+    my ($attr, $idx) = @_;
+
+    return if $attr->can('is_lazy') && $attr->is_lazy;
+    my $default = $self->_inline_default_value($attr, $idx);
+    return unless $default;
+
+    my @initial_value = (
+        'my $default = ' . $default . ';',
+        $attr->_inline_set_value(
+            '$instance',
+            '$default',
+            '$type_constraint_bodies[' . $idx . ']',
+            '$type_coercions[' . $idx . ']',
+            '$type_constraint_messages[' . $idx . ']',
+            'for constructor',
+        ),
+    );
+
+    push @initial_value, (
+        '$attrs->[' . $idx . ']->set_initial_value(',
+            '$instance,',
+            $attr->_inline_instance_get('$instance'),
+        ');',
+    ) if $attr->has_initializer;
+
+    return @initial_value;
+}
+
+sub _inline_extra_init {
+    my $self = shift;
+    return (
+        $self->_inline_triggers,
+        $self->_inline_BUILDALL,
+    );
+}
+
+sub _inline_triggers {
+    my $self = shift;
+    my @trigger_calls;
+
+    my @attrs = sort { $a->name cmp $b->name } $self->get_all_attributes;
+    for my $i (0 .. $#attrs) {
+        my $attr = $attrs[$i];
+
+        next unless $attr->can('has_trigger') && $attr->has_trigger;
+
+        my $init_arg = $attr->init_arg;
+        next unless defined $init_arg;
+
+        push @trigger_calls,
+            'if (exists $params->{\'' . $init_arg . '\'}) {',
+                '$triggers->[' . $i . ']->(',
+                    '$instance,',
+                    $attr->_inline_instance_get('$instance') . ',',
+                ');',
+            '}';
+    }
+
+    return @trigger_calls;
+}
+
+sub _inline_BUILDALL {
+    my $self = shift;
+
+    my @methods = reverse $self->find_all_methods_by_name('BUILD');
+    my @BUILD_calls;
+
+    foreach my $method (@methods) {
+        push @BUILD_calls,
+            '$instance->' . $method->{class} . '::BUILD($params);';
+    }
+
+    return @BUILD_calls;
+}
+
+sub _eval_environment {
+    my $self = shift;
+
+    my @attrs = sort { $a->name cmp $b->name } $self->get_all_attributes;
+
+    my $triggers = [
+        map { $_->can('has_trigger') && $_->has_trigger ? $_->trigger : undef }
+            @attrs
+    ];
+
+    # We need to check if the attribute ->can('type_constraint')
+    # since we may be trying to immutabilize a Moose meta class,
+    # which in turn has attributes which are Class::MOP::Attribute
+    # objects, rather than Moose::Meta::Attribute. And
+    # Class::MOP::Attribute attributes have no type constraints.
+    # However we need to make sure we leave an undef value there
+    # because the inlined code is using the index of the attributes
+    # to determine where to find the type constraint
+
+    my @type_constraints = map {
+        $_->can('type_constraint') ? $_->type_constraint : undef
+    } @attrs;
+
+    my @type_constraint_bodies = map {
+        defined $_ ? $_->_compiled_type_constraint : undef;
+    } @type_constraints;
+
+    my @type_coercions = map {
+        defined $_ && $_->has_coercion
+            ? $_->coercion->_compiled_type_coercion
+            : undef
+    } @type_constraints;
+
+    my @type_constraint_messages = map {
+        defined $_
+            ? ($_->has_message ? $_->message : $_->_default_message)
+            : undef
+    } @type_constraints;
+
+    return {
+        %{ $self->SUPER::_eval_environment },
+        ((any { defined && $_->has_initializer } @attrs)
+            ? ('$attrs' => \[@attrs])
+            : ()),
+        '$triggers' => \$triggers,
+        '@type_coercions' => \@type_coercions,
+        '@type_constraint_bodies' => \@type_constraint_bodies,
+        '@type_constraint_messages' => \@type_constraint_messages,
+        ( map { defined($_) ? %{ $_->inline_environment } : () }
+              @type_constraints ),
+        # pretty sure this is only going to be closed over if you use a custom
+        # error class at this point, but we should still get rid of this
+        # at some point
+        '$meta'  => \$self,
+    };
 }
 
 sub superclasses {
@@ -361,224 +627,24 @@ sub _base_metaclasses {
     );
 }
 
-sub _find_common_base {
-    my $self = shift;
-    my ($meta1, $meta2) = map { Class::MOP::class_of($_) } @_;
-    return unless defined $meta1 && defined $meta2;
-
-    # FIXME? This doesn't account for multiple inheritance (not sure
-    # if it needs to though). For example, if somewhere in $meta1's
-    # history it inherits from both ClassA and ClassB, and $meta2
-    # inherits from ClassB & ClassA, does it matter? And what crazy
-    # fool would do that anyway?
-
-    my %meta1_parents = map { $_ => 1 } $meta1->linearized_isa;
-
-    return first { $meta1_parents{$_} } $meta2->linearized_isa;
-}
-
-sub _get_ancestors_until {
-    my $self = shift;
-    my ($start_name, $until_name) = @_;
-
-    my @ancestor_names;
-    for my $ancestor_name (Class::MOP::class_of($start_name)->linearized_isa) {
-        last if $ancestor_name eq $until_name;
-        push @ancestor_names, $ancestor_name;
-    }
-    return @ancestor_names;
-}
-
-sub _is_role_only_subclass {
-    my $self = shift;
-    my ($meta_name) = @_;
-    my $meta = Class::MOP::Class->initialize($meta_name);
-    my @parent_names = $meta->superclasses;
-
-    # XXX: don't feel like messing with multiple inheritance here... what would
-    # that even do?
-    return unless @parent_names == 1;
-    my ($parent_name) = @parent_names;
-    my $parent_meta = Class::MOP::Class->initialize($parent_name);
-
-    my @roles = $meta->can('calculate_all_roles_with_inheritance')
-                    ? $meta->calculate_all_roles_with_inheritance
-                    : ();
-
-    # loop over all methods that are a part of the current class
-    # (not inherited)
-    for my $method (map { $meta->get_method($_) } $meta->get_method_list) {
-        # always ignore meta
-        next if $method->name eq 'meta';
-        # we'll deal with attributes below
-        next if $method->can('associated_attribute');
-        # if the method comes from a role we consumed, ignore it
-        next if $meta->can('does_role')
-             && $meta->does_role($method->original_package_name);
-        # FIXME - this really isn't right. Just because a modifier is
-        # defined in a role doesn't mean it isn't _also_ defined in the
-        # subclass.
-        next if $method->isa('Class::MOP::Method::Wrapped')
-             && (
-                 (!scalar($method->around_modifiers)
-               || any { $_->has_around_method_modifiers($method->name) } @roles)
-              && (!scalar($method->before_modifiers)
-               || any { $_->has_before_method_modifiers($method->name) } @roles)
-              && (!scalar($method->after_modifiers)
-               || any { $_->has_after_method_modifiers($method->name) } @roles)
-                );
-
-        return 0;
-    }
-
-    # loop over all attributes that are a part of the current class
-    # (not inherited)
-    # FIXME - this really isn't right. Just because an attribute is
-    # defined in a role doesn't mean it isn't _also_ defined in the
-    # subclass.
-    for my $attr (map { $meta->get_attribute($_) } $meta->get_attribute_list) {
-        next if any { $_->has_attribute($attr->name) } @roles;
-
-        return 0;
-    }
-
-    return 1;
-}
-
-sub _can_fix_class_metaclass_incompatibility_by_role_reconciliation {
-    my $self = shift;
-    my ($super_meta) = @_;
-
-    my $super_meta_name = $super_meta->_real_ref_name;
-
-    return $self->_classes_differ_by_roles_only(
-        blessed($self),
-        $super_meta_name,
-        'Moose::Meta::Class',
-    );
-}
-
-sub _can_fix_single_metaclass_incompatibility_by_role_reconciliation {
-    my $self = shift;
-    my ($metaclass_type, $super_meta) = @_;
-
-    my $class_specific_meta_name = $self->$metaclass_type;
-    return unless $super_meta->can($metaclass_type);
-    my $super_specific_meta_name = $super_meta->$metaclass_type;
-    my %metaclasses = $self->_base_metaclasses;
-
-    return $self->_classes_differ_by_roles_only(
-        $class_specific_meta_name,
-        $super_specific_meta_name,
-        $metaclasses{$metaclass_type},
-    );
-}
-
-sub _classes_differ_by_roles_only {
-    my $self = shift;
-    my ( $self_meta_name, $super_meta_name, $expected_ancestor ) = @_;
-
-    my $common_base_name
-        = $self->_find_common_base( $self_meta_name, $super_meta_name );
-
-    # If they're not both moose metaclasses, and the cmop fixing couldn't do
-    # anything, there's nothing more we can do. The $expected_ancestor should
-    # always be a Moose metaclass name like Moose::Meta::Class or
-    # Moose::Meta::Attribute.
-    return unless defined $common_base_name;
-    return unless $common_base_name->isa($expected_ancestor);
-
-    my @super_meta_name_ancestor_names
-        = $self->_get_ancestors_until( $super_meta_name, $common_base_name );
-    my @class_meta_name_ancestor_names
-        = $self->_get_ancestors_until( $self_meta_name, $common_base_name );
-
-    return
-        unless all { $self->_is_role_only_subclass($_) }
-        @super_meta_name_ancestor_names,
-        @class_meta_name_ancestor_names;
-
-    return 1;
-}
-
-sub _role_differences {
-    my $self = shift;
-    my ($class_meta_name, $super_meta_name) = @_;
-    my @super_role_metas = $super_meta_name->meta->can('calculate_all_roles_with_inheritance')
-                         ? $super_meta_name->meta->calculate_all_roles_with_inheritance
-                         : ();
-    my @role_metas       = $class_meta_name->meta->can('calculate_all_roles_with_inheritance')
-                         ? $class_meta_name->meta->calculate_all_roles_with_inheritance
-                         : ();
-    my @differences;
-    for my $role_meta (@role_metas) {
-        push @differences, $role_meta
-            unless any { $_->name eq $role_meta->name } @super_role_metas;
-    }
-    return @differences;
-}
-
-sub _reconcile_roles_for_metaclass {
-    my $self = shift;
-    my ($class_meta_name, $super_meta_name) = @_;
-
-    my @role_differences = $self->_role_differences(
-        $class_meta_name, $super_meta_name,
-    );
-
-    # handle the case where we need to fix compatibility between a class and
-    # its parent, but all roles in the class are already also done by the
-    # parent
-    # see t/050/054.t
-    return Class::MOP::class_of($super_meta_name)
-        unless @role_differences;
-
-    return Moose::Meta::Class->create_anon_class(
-        superclasses => [$super_meta_name],
-        roles        => \@role_differences,
-        cache        => 1,
-    );
-}
-
-sub _can_fix_metaclass_incompatibility_by_role_reconciliation {
-    my $self = shift;
-    my ($super_meta) = @_;
-
-    return 1 if $self->_can_fix_class_metaclass_incompatibility_by_role_reconciliation($super_meta);
-
-    my %base_metaclass = $self->_base_metaclasses;
-    for my $metaclass_type (keys %base_metaclass) {
-        next unless defined $self->$metaclass_type;
-        return 1 if $self->_can_fix_single_metaclass_incompatibility_by_role_reconciliation($metaclass_type, $super_meta);
-    }
-
-    return;
-}
-
-sub _can_fix_metaclass_incompatibility {
-    my $self = shift;
-    return 1 if $self->_can_fix_metaclass_incompatibility_by_role_reconciliation(@_);
-    return $self->SUPER::_can_fix_metaclass_incompatibility(@_);
-}
-
 sub _fix_class_metaclass_incompatibility {
     my $self = shift;
     my ($super_meta) = @_;
 
     $self->SUPER::_fix_class_metaclass_incompatibility(@_);
 
-    if ($self->_can_fix_class_metaclass_incompatibility_by_role_reconciliation($super_meta)) {
+    if ($self->_class_metaclass_can_be_made_compatible($super_meta)) {
         ($self->is_pristine)
             || confess "Can't fix metaclass incompatibility for "
                      . $self->name
                      . " because it is not pristine.";
         my $super_meta_name = $super_meta->_real_ref_name;
-        my $class_meta_subclass_meta = $self->_reconcile_roles_for_metaclass(blessed($self), $super_meta_name);
-        my $new_self = $class_meta_subclass_meta->name->reinitialize(
+        my $class_meta_subclass_meta_name = Moose::Util::_reconcile_roles_for_metaclass(blessed($self), $super_meta_name);
+        my $new_self = $class_meta_subclass_meta_name->reinitialize(
             $self->name,
         );
 
-        $self->_replace_self( $new_self, $class_meta_subclass_meta->name );
+        $self->_replace_self( $new_self, $class_meta_subclass_meta_name );
     }
 }
 
@@ -588,23 +654,22 @@ sub _fix_single_metaclass_incompatibility {
 
     $self->SUPER::_fix_single_metaclass_incompatibility(@_);
 
-    if ($self->_can_fix_single_metaclass_incompatibility_by_role_reconciliation($metaclass_type, $super_meta)) {
+    if ($self->_single_metaclass_can_be_made_compatible($super_meta, $metaclass_type)) {
         ($self->is_pristine)
             || confess "Can't fix metaclass incompatibility for "
                      . $self->name
                      . " because it is not pristine.";
         my $super_meta_name = $super_meta->_real_ref_name;
-        my $class_specific_meta_subclass_meta = $self->_reconcile_roles_for_metaclass($self->$metaclass_type, $super_meta->$metaclass_type);
+        my $class_specific_meta_subclass_meta_name = Moose::Util::_reconcile_roles_for_metaclass($self->$metaclass_type, $super_meta->$metaclass_type);
         my $new_self = $super_meta->reinitialize(
             $self->name,
-            $metaclass_type => $class_specific_meta_subclass_meta->name,
+            $metaclass_type => $class_specific_meta_subclass_meta_name,
         );
 
         $self->_replace_self( $new_self, $super_meta_name );
     }
 }
 
-
 sub _replace_self {
     my $self      = shift;
     my ( $new_self, $new_class)   = @_;
@@ -615,8 +680,9 @@ sub _replace_self {
     # We need to replace the cached metaclass instance or else when it goes
     # out of scope Class::MOP::Class destroy's the namespace for the
     # metaclass's class, causing much havoc.
+    my $weaken = Class::MOP::metaclass_is_weak( $self->name );
     Class::MOP::store_metaclass_by_name( $self->name, $self );
-    Class::MOP::weaken_metaclass( $self->name ) if $self->is_anon_class;
+    Class::MOP::weaken_metaclass( $self->name ) if $weaken;
 }
 
 sub _process_attribute {
@@ -653,6 +719,52 @@ sub _process_inherited_attribute {
     }
 }
 
+# reinitialization support
+
+sub _restore_metaobjects_from {
+    my $self = shift;
+    my ($old_meta) = @_;
+
+    $self->SUPER::_restore_metaobjects_from($old_meta);
+
+    for my $role ( @{ $old_meta->roles } ) {
+        $self->add_role($role);
+    }
+
+    for my $application ( @{ $old_meta->_get_role_applications } ) {
+        $application->class($self);
+        $self->add_role_application ($application);
+    }
+}
+
+## Immutability
+
+sub _immutable_options {
+    my ( $self, @args ) = @_;
+
+    $self->SUPER::_immutable_options(
+        inline_destructor => 1,
+
+        # Moose always does this when an attribute is created
+        inline_accessors => 0,
+
+        @args,
+    );
+}
+
+sub _fixup_attributes_after_rebless {
+    my $self = shift;
+    my ($instance, $rebless_from, %params) = @_;
+
+    $self->SUPER::_fixup_attributes_after_rebless(
+        $instance,
+        $rebless_from,
+        %params
+    );
+
+    $self->_call_all_triggers( $instance, \%params );
+}
+
 ## -------------------------------------------------
 
 our $error_level;
@@ -663,11 +775,22 @@ sub throw_error {
     $self->raise_error($self->create_error(@args));
 }
 
+sub _inline_throw_error {
+    my ( $self, @args ) = @_;
+    $self->_inline_raise_error($self->_inline_create_error(@args));
+}
+
 sub raise_error {
     my ( $self, @args ) = @_;
     die @args;
 }
 
+sub _inline_raise_error {
+    my ( $self, $message ) = @_;
+
+    return 'die ' . $message;
+}
+
 sub create_error {
     my ( $self, @args ) = @_;
 
@@ -693,9 +816,41 @@ sub create_error {
     );
 }
 
+sub _inline_create_error {
+    my ( $self, $msg, $args ) = @_;
+    # XXX ignore $args for now, nothing currently uses it anyway
+
+    require Carp::Heavy;
+
+    my %args = (
+        metaclass  => $self,
+        last_error => $@,
+        message    => $msg,
+    );
+
+    my $class = ref $self ? $self->error_class : "Moose::Error::Default";
+
+    Class::MOP::load_class($class);
+
+    # don't check inheritance here - the intention is that the class needs
+    # to provide a non-inherited inlining method, because falling back to
+    # the default inlining method is most likely going to be wrong
+    # yes, this is a huge hack, but so is the entire error system, so.
+    return '$meta->create_error(' . $msg . ', ' . $args . ');'
+        unless $class->meta->has_method('_inline_new');
+
+    $class->_inline_new(
+        # XXX ignore this for now too
+        # Carp::caller_info($args{depth}),
+        %args
+    );
+}
+
 1;
 
-__END__
+# ABSTRACT: The Moose metaclass
+
+
 
 =pod
 
@@ -703,6 +858,10 @@ __END__
 
 Moose::Meta::Class - The Moose metaclass
 
+=head1 VERSION
+
+version 2.0200
+
 =head1 DESCRIPTION
 
 This class is a subclass of L<Class::MOP::Class> that provides
@@ -768,8 +927,8 @@ This overrides the parent's method to add a few options. Specifically,
 it uses the Moose-specific constructor and destructor classes, and
 enables inlining the destructor.
 
-Also, since Moose always inlines attributes, it sets the
-C<inline_accessors> option to false.
+Since Moose always inlines attributes, it sets the C<inline_accessors> option
+to false.
 
 =item B<< $metaclass->new_object(%params) >>
 
@@ -840,8 +999,8 @@ be provided as a hash reference.
 
 =item B<< $metaclass->destructor_class($class_name) >>
 
-These are the names of classes used when making a class
-immutable. These default to L<Moose::Meta::Method::Constructor> and
+These are the names of classes used when making a class immutable. These
+default to L<Moose::Meta::Method::Constructor> and
 L<Moose::Meta::Method::Destructor> respectively. These accessors are
 read-write, so you can use them to change the class name.
 
@@ -863,16 +1022,18 @@ See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
-Stevan Little E<lt>stevan@iinteractive.comE<gt>
+Stevan Little <stevan@iinteractive.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2010 by Infinity Interactive, Inc.
+This software is copyright (c) 2011 by Infinity Interactive, Inc..
 
-L<http://www.iinteractive.com>
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
 
+
+__END__
+
+
