@@ -1,13 +1,15 @@
 
 package Moose::Meta::Role::Method;
+BEGIN {
+  $Moose::Meta::Role::Method::AUTHORITY = 'cpan:STEVAN';
+}
+BEGIN {
+  $Moose::Meta::Role::Method::VERSION = '2.0200';
+}
 
 use strict;
 use warnings;
 
-our $VERSION   = '1.23';
-$VERSION = eval $VERSION;
-our $AUTHORITY = 'cpan:STEVAN';
-
 use base 'Moose::Meta::Method';
 
 sub _make_compatible_with {
@@ -23,7 +25,9 @@ sub _make_compatible_with {
 
 1;
 
-__END__
+# ABSTRACT: A Moose Method metaclass for Roles
+
+
 
 =pod
 
@@ -31,6 +35,10 @@ __END__
 
 Moose::Meta::Role::Method - A Moose Method metaclass for Roles
 
+=head1 VERSION
+
+version 2.0200
+
 =head1 DESCRIPTION
 
 This is primarily used to mark methods coming from a role
@@ -43,15 +51,17 @@ See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
-Stevan Little E<lt>stevan@iinteractive.comE<gt>
+Stevan Little <stevan@iinteractive.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2010 by Infinity Interactive, Inc.
+This software is copyright (c) 2011 by Infinity Interactive, Inc..
 
-L<http://www.iinteractive.com>
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+
+__END__
+
