@@ -1,5 +1,11 @@
 
 package Moose::Meta::Role::Method::Required;
+BEGIN {
+  $Moose::Meta::Role::Method::Required::AUTHORITY = 'cpan:STEVAN';
+}
+BEGIN {
+  $Moose::Meta::Role::Method::Required::VERSION = '2.0200';
+}
 
 use strict;
 use warnings;
@@ -10,23 +16,22 @@ use overload '""'     => sub { shift->name },   # stringify to method name
 
 use base qw(Class::MOP::Object);
 
-our $VERSION   = '1.23';
-$VERSION = eval $VERSION;
-our $AUTHORITY = 'cpan:STEVAN';
-
 # This is not a Moose::Meta::Role::Method because it has no implementation, it
 # is just a name
 
 __PACKAGE__->meta->add_attribute('name' => (
     reader   => 'name',
     required => 1,
+    Class::MOP::_definition_context(),
 ));
 
 sub new { shift->_new(@_) }
 
 1;
 
-__END__
+# ABSTRACT: A Moose metaclass for required methods in Roles
+
+
 
 =pod
 
@@ -34,6 +39,10 @@ __END__
 
 Moose::Meta::Role::Method::Required - A Moose metaclass for required methods in Roles
 
+=head1 VERSION
+
+version 2.0200
+
 =head1 DESCRIPTION
 
 =head1 INHERITANCE
@@ -70,15 +79,17 @@ See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
-Stevan Little E<lt>stevan@iinteractive.comE<gt>
+Stevan Little <stevan@iinteractive.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2010 by Infinity Interactive, Inc.
+This software is copyright (c) 2011 by Infinity Interactive, Inc..
 
-L<http://www.iinteractive.com>
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+
+__END__
+
