@@ -1,5 +1,11 @@
 
 package Moose::Meta::TypeCoercion;
+BEGIN {
+  $Moose::Meta::TypeCoercion::AUTHORITY = 'cpan:STEVAN';
+}
+BEGIN {
+  $Moose::Meta::TypeCoercion::VERSION = '2.0200';
+}
 
 use strict;
 use warnings;
@@ -8,25 +14,24 @@ use metaclass;
 use Moose::Meta::Attribute;
 use Moose::Util::TypeConstraints ();
 
-our $VERSION   = '1.09';
-$VERSION = eval $VERSION;
-our $AUTHORITY = 'cpan:STEVAN';
-
 __PACKAGE__->meta->add_attribute('type_coercion_map' => (
     reader  => 'type_coercion_map',
-    default => sub { [] }
+    default => sub { [] },
+    Class::MOP::_definition_context(),
 ));
 
 __PACKAGE__->meta->add_attribute(
     Moose::Meta::Attribute->new('type_constraint' => (
         reader   => 'type_constraint',
-        weak_ref => 1
+        weak_ref => 1,
+        Class::MOP::_definition_context(),
     ))
 );
 
 # private accessor
 __PACKAGE__->meta->add_attribute('compiled_type_coercion' => (
-    accessor => '_compiled_type_coercion'
+    accessor => '_compiled_type_coercion',
+    Class::MOP::_definition_context(),
 ));
 
 sub new {
@@ -99,7 +104,9 @@ sub coerce { $_[0]->_compiled_type_coercion->($_[1]) }
 
 1;
 
-__END__
+# ABSTRACT: The Moose Type Coercion metaclass
+
+
 
 =pod
 
@@ -107,6 +114,10 @@ __END__
 
 Moose::Meta::TypeCoercion - The Moose Type Coercion metaclass
 
+=head1 VERSION
+
+version 2.0200
+
 =head1 DESCRIPTION
 
 A type coercion object is basically a mapping of one or more type
@@ -178,15 +189,17 @@ See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
-Stevan Little E<lt>stevan@iinteractive.comE<gt>
+Stevan Little <stevan@iinteractive.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2010 by Infinity Interactive, Inc.
+This software is copyright (c) 2011 by Infinity Interactive, Inc..
 
-L<http://www.iinteractive.com>
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+
+__END__
+
