@@ -1,14 +1,16 @@
 package oose;
+BEGIN {
+  $oose::AUTHORITY = 'cpan:STEVAN';
+}
+BEGIN {
+  $oose::VERSION = '2.0200';
+}
 
 use strict;
 use warnings;
 
 use Class::MOP;
 
-our $VERSION   = '1.09';
-$VERSION = eval $VERSION;
-our $AUTHORITY = 'cpan:STEVAN';
-
 BEGIN {
     my $package;
     sub import {
@@ -23,7 +25,9 @@ BEGIN {
 
 1;
 
-__END__
+# ABSTRACT: syntactic sugar to make Moose one-liners easier
+
+
 
 =pod
 
@@ -31,6 +35,10 @@ __END__
 
 oose - syntactic sugar to make Moose one-liners easier
 
+=head1 VERSION
+
+version 2.0200
+
 =head1 SYNOPSIS
 
   # create a Moose class on the fly ...
@@ -46,7 +54,8 @@ oose - syntactic sugar to make Moose one-liners easier
 
 =head1 DESCRIPTION
 
-oose.pm is a simple source filter that adds C<package $name; use Moose;>
+oose.pm is a simple source filter that adds
+C<package $name; use Moose; use Moose::Util::TypeConstraints;>
 to the beginning of your script and was entirely created because typing
 C<perl -e'package Foo; use Moose; ...'> was annoying me.
 
@@ -58,7 +67,8 @@ oose provides exactly one method and it's automatically called by perl:
 
 =item B<import($package)>
 
-Pass a package name to import to be used by the source filter.
+Pass a package name to import to be used by the source filter. The
+package defaults to C<Class> if none is given.
 
 =back
 
@@ -76,15 +86,17 @@ See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
-Chris Prather  C<< <chris@prather.org> >>
+Stevan Little <stevan@iinteractive.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2007-2009 by Infinity Interactive, Inc.
+This software is copyright (c) 2011 by Infinity Interactive, Inc..
 
-L<http://www.iinteractive.com>
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+
+__END__
+
